/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
@file:Suppress("unused")

package edu.ucsc.its.temerity.extensions.log

import co.touchlab.kermit.CommonWriter
import co.touchlab.kermit.LogWriter
import co.touchlab.kermit.Severity
import co.touchlab.kermit.StaticConfig
import org.slf4j.ILoggerFactory
import org.slf4j.helpers.BasicMarkerFactory
import org.slf4j.helpers.NOPMDCAdapter

internal class KermitServiceProvider : org.slf4j.spi.SLF4JServiceProvider {
  private val markerFactory = BasicMarkerFactory()
  private val mdcAdapter = NOPMDCAdapter()
  override fun getLoggerFactory() = ILoggerFactory {
    Slf4jKermitLogger(it, config)
  }

  override fun getMarkerFactory() = markerFactory

  override fun getMDCAdapter() = mdcAdapter

  override fun getRequestedApiVersion() = "2.0.99"

  override fun initialize() = Unit

  companion object {
    private val writers = mutableListOf<LogWriter>().apply {
      add(CommonWriter())
    }
    var config: StaticConfig = StaticConfig(logWriterList = writers)

    var minSeverity: Severity
      get() = config.minSeverity
      set(value) {
        config = config.copy(minSeverity = value)
      }

    fun addWriter(writer: LogWriter) {
      writers.add(writer)
      config = config.copy(logWriterList = writers)
    }

    fun setWriters(vararg writer: LogWriter) {
      writers.clear()
      writers.addAll(writer)
      config = config.copy(logWriterList = writers)
    }
  }
}
